using System;
using System.Reflection;
using System.Collections;
using Server;
using Server.Targeting;
using CPA = Server.CommandPropertyAttribute;

namespace Server.Scripts.Commands
{
	public class ObjectConditional
	{
		private Type m_Type;
		private PropertyCondition[][] m_Conditions;

		private static Type typeofItem = typeof( Item );
		private static Type typeofMobile = typeof( Mobile );

		public bool IsItem
		{
			get{ return ( m_Type == null || m_Type == typeofItem || m_Type.IsSubclassOf( typeofItem ) ); }
		}

		public bool IsMobile
		{
			get{ return ( m_Type == null || m_Type == typeofMobile || m_Type.IsSubclassOf( typeofMobile ) ); }
		}

		public static readonly ObjectConditional Empty = new ObjectConditional( null, new PropertyCondition[0][] );

		public static void Initialize()
		{
			Server.Commands.Register( "Condition", AccessLevel.Administrator,new CommandEventHandler( Condition_OnCommand ) );
		}

		[Usage( "Condition <condition>" )]
		[Description( "Checks that the given condition matches a targeted object." )]
		private static void Condition_OnCommand( CommandEventArgs e )
		{
			e.Mobile.BeginTarget( -1, false, TargetFlags.None, new TargetStateCallback( Condition_OnTarget ), e.Arguments );
		}

		private static void Condition_OnTarget( Mobile from, object obj, object state )
		{
			string[] args = (string[])state;
			ObjectConditional cond = Parse( from, ref args );

			from.SendMessage( cond.CheckCondition( obj ).ToString() );
		}

		public bool CheckCondition( object obj )
		{
			if ( m_Type == null )
				return true; // null type means no condition

			Type objType = obj.GetType();

			if ( objType != m_Type && !objType.IsSubclassOf( m_Type ) )
				return false;

			bool res = true;

			for ( int i = 0; res && i < m_Conditions.Length; ++i )
			{
				PropertyCondition[] conditions = m_Conditions[i];

				for ( int j = 0; res && j < conditions.Length; ++j )
					res = conditions[j].CheckCondition( obj );
			}

			return res;
		}

		public static ObjectConditional Parse( Mobile from, ref string[] args )
		{
			string[] conditionArgs = null;

			for ( int i = 0; i < args.Length; ++i )
			{
				if ( Insensitive.Equals( args[i], "where" ) )
				{
					string[] origArgs = args;

					args = new string[i];

					for ( int j = 0; j < args.Length; ++j )
						args[j] = origArgs[j];

					conditionArgs = new string[origArgs.Length - i - 1];

					for ( int j = 0; j < conditionArgs.Length; ++j )
						conditionArgs[j] = origArgs[i + j + 1];

					break;
				}
			}

			if ( conditionArgs == null || conditionArgs.Length == 0 )
				return ObjectConditional.Empty;

			int index = 0;

			Type type = ScriptCompiler.FindTypeByName( conditionArgs[index++], true );

			if ( type == null )
				throw new Exception( String.Format( "No type with that name ({0}) was found.", conditionArgs[0] ) );

			PropertyInfo[] props = type.GetProperties();

			ArrayList allConditions = new ArrayList();
			ArrayList currentConditions = null;

			while ( index < conditionArgs.Length )
			{
				string cur = conditionArgs[index];

				bool logicalNot = false;

				if ( Insensitive.Equals( cur, "not" ) || cur == "!" )
				{
					logicalNot = true;
					++index;

					if ( index >= conditionArgs.Length )
						throw new Exception( "Improperly formatted object conditional." );
				}
				else if ( Insensitive.Equals( cur, "or" ) || cur == "||" )
				{
					if ( currentConditions != null )
					{
						allConditions.Add( currentConditions );
						currentConditions = null;
					}

					continue;
				}

				string prop = conditionArgs[index++];

				if ( index >= conditionArgs.Length )
					throw new Exception( "Improperly formatted object conditional." );

				string oper = conditionArgs[index++];

				if ( index >= conditionArgs.Length )
					throw new Exception( "Improperly formatted object conditional." );

				string arg = conditionArgs[index++];

				if ( currentConditions == null )
					currentConditions = new ArrayList();

				currentConditions.Add( new PropertyCondition( from, type, props, prop, oper, arg, logicalNot ) );
			}

			if ( currentConditions != null )
				allConditions.Add( currentConditions );

			PropertyCondition[][] conditions = new PropertyCondition[allConditions.Count][];

			for ( int i = 0; i < conditions.Length; ++i )
				conditions[i] = (PropertyCondition[])(((ArrayList)allConditions[i]).ToArray( typeof( PropertyCondition ) ));

			return new ObjectConditional( type, conditions );
		}

		public ObjectConditional( Type type, PropertyCondition[][] conditions )
		{
			m_Type = type;
			m_Conditions = conditions;
		}
	}

	public enum ConditionOperator
	{
		// all
		Equality,
		Inequality,

		// numerical
		Greater,
		GreaterEqual,
		Lesser,
		LesserEqual,

		// strings
		EqualityInsensitive,
		InequalityInsensitive,
		StartsWith,
		StartsWithInsensitive,
		EndsWith,
		EndsWithInsensitive,
		Contains,
		ContainsInsensitive
	}

	public class PropertyCondition
	{
		private PropertyInfo m_Property;
		private ConditionOperator m_Operator;
		private object m_Argument;
		private bool m_LogicalNot;

		public PropertyCondition( Mobile from, Type type, PropertyInfo[] props, string prop, string oper, string arg, bool logicalNot )
		{
			m_LogicalNot = logicalNot;

			for ( int i = 0; i < props.Length; ++i )
			{
				PropertyInfo check = props[i];

				if ( !Insensitive.Equals( check.Name, prop ) )
					continue;

				m_Property = check;
				break;
			}

			if ( m_Property == null )
				throw new Exception( String.Format( "No property with the name ({0}) was found on type ({1}).", prop, type.Name ) );

			CPA attr = Properties.GetCPA( m_Property );

			if ( attr == null )
				throw new Exception( String.Format( "No property with the name ({0}) was found on type ({1}).", prop, type.Name ) );

			if ( from.AccessLevel < attr.ReadLevel )
				throw new Exception( String.Format( "Getting this property ({0}) requires at least {1} access level.", prop, Mobile.GetAccessLevelName( attr.ReadLevel ) ) );

			string error = Properties.ConstructFromString( m_Property.PropertyType, null, arg, ref m_Argument );

			if ( error != null )
				throw new Exception( error );

			if ( m_Argument != null && !(m_Argument is IComparable) )
				throw new Exception( String.Format( "This property ({0}) is not comparable.", prop ) );

			switch ( oper )
			{
				case "=":
				case "==":
				case "is": m_Operator = ConditionOperator.Equality; break;

				case "!=": m_Operator = ConditionOperator.Inequality; break;

				case ">": m_Operator = ConditionOperator.Greater; break;
				case "<": m_Operator = ConditionOperator.Lesser; break;

				case ">=": m_Operator = ConditionOperator.GreaterEqual; break;
				case "<=": m_Operator = ConditionOperator.LesserEqual; break;

				case "==~":
				case "~==":
				case "=~":
				case "~=":
				case "is~":
				case "~is": m_Operator = ConditionOperator.EqualityInsensitive; break;

				case "!=~":
				case "~!=": m_Operator = ConditionOperator.InequalityInsensitive; break;

				case "starts": m_Operator = ConditionOperator.StartsWith; break;

				case "starts~":
				case "~starts": m_Operator = ConditionOperator.StartsWithInsensitive; break;

				case "ends": m_Operator = ConditionOperator.EndsWith; break;

				case "ends~":
				case "~ends": m_Operator = ConditionOperator.EndsWithInsensitive; break;

				case "contains": m_Operator = ConditionOperator.Contains; break;

				case "contains~":
				case "~contains": m_Operator = ConditionOperator.ContainsInsensitive; break;
			}
		}

		public string AsString( object obj )
		{
			if ( obj == null )
				return "";

			if ( obj is string )
				return (string)obj;

			return obj.ToString();
		}

		public int CompareWith( object obj )
		{
			if ( obj == null && m_Argument == null )
				return 0;

			if ( obj == null )
				return -1;

			if ( m_Argument == null )
				return 1;

			if ( !(obj is IComparable) )
				throw new Exception( String.Format( "This property ({0}) returned an incomparable object of type {1}.", m_Property.Name, obj.GetType() ) );

			return ((IComparable)obj).CompareTo( m_Argument );
		}

		public bool CheckCondition( object obj )
		{
			bool ret;

			object current = m_Property.GetValue( obj, null );

			switch ( m_Operator )
			{
				case ConditionOperator.Equality:
					ret = ( CompareWith( current ) == 0 ); break;

				case ConditionOperator.Inequality:
					ret = ( CompareWith( current ) != 0 ); break;

				case ConditionOperator.Greater:
					ret = ( CompareWith( current ) > 0 ); break;

				case ConditionOperator.GreaterEqual:
					ret = ( CompareWith( current ) >= 0 ); break;

				case ConditionOperator.Lesser:
					ret = ( CompareWith( current ) < 0 ); break;

				case ConditionOperator.LesserEqual:
					ret = ( CompareWith( current ) <= 0 ); break;

				case ConditionOperator.EqualityInsensitive:
					ret = ( Insensitive.Equals( AsString( current ), AsString( m_Argument ) ) ); break;

				case ConditionOperator.InequalityInsensitive:
					ret = ( !Insensitive.Equals( AsString( current ), AsString( m_Argument ) ) ); break;

				case ConditionOperator.StartsWith:
					ret = ( AsString( current ).StartsWith( AsString( m_Argument ) ) ); break;

				case ConditionOperator.StartsWithInsensitive:
					ret = ( Insensitive.StartsWith( AsString( current ), AsString( m_Argument ) ) ); break;

				case ConditionOperator.EndsWith:
					ret = ( AsString( current ).EndsWith( AsString( m_Argument ) ) ); break;

				case ConditionOperator.EndsWithInsensitive:
					ret = ( Insensitive.EndsWith( AsString( current ), AsString( m_Argument ) ) ); break;

				case ConditionOperator.Contains:
					ret = ( AsString( current ).IndexOf( AsString( m_Argument ) ) >= 0 ); break;

				case ConditionOperator.ContainsInsensitive:
					ret = ( Insensitive.Contains( AsString( current ), AsString( m_Argument ) ) ); break;

				default: return false;
			}

			if ( m_LogicalNot )
				ret = !ret;

			return ret;
		}
	}
}